<?php

require_once 'DefaultController.class.php';

class MK_DashboardController extends MK_DefaultController
{

	public function _init()
	{
		parent::_init();
		$this->getView()->getHead()->prependTitle( 'Dashboard' );
	}
	
	public function sectionActionLog()
	{
		if( MK_Request::getParam('method') == 'delete' )
		{
			$action_log_module = MK_RecordModuleManager::getFromType('action_log');
			$action_log_module->flush();
		}
		$this->getView()->redirect(array('controller' => 'dashboard', 'section' => 'index'));
	}
	
	public function sectionInstalledComponents()
	{
		$config = MK_Config::getInstance();

		$components = MK_ComponentManager::getComponents( MK_ComponentManager::TYPE_OPTIONAL );
		$installed_components = $config->db->components;

		$form_structure = array(
			'components' => array(
				'label' => 'Components',
				'type' => 'checkbox-multiple',
				'options' => $components
			),
			'submit' => array(
				'type' => 'submit',
				'attributes' => array(
					'value' => 'Install Selected Modules'
				)
			)
		);

		$form_settings = array(
			'attributes' => array(
				'class' => 'standard clear-fix large'
			)
		);
		
		$form = new MK_Form($form_structure, $form_settings);

		$this->getView()->form = $form->render();

		if( $form->isSuccessful() )
		{
			$this->getView()->setDisplayPath('dashboard/installed-components-processed');		

			$fields = $form->getFields();
			
			$selected_components = $form->getField('components')->getValue();
			
			$messages = array();
			
			try
			{
				foreach( $selected_components as $selected_component )
				{
					$details = MK_ComponentManager::installComponent( $selected_component );
					$messages[] = new MK_Message('success', "The <strong>".$details['name']."</strong> component was installed successfully.");
				}
				$messages[] = new MK_Message('success', "The selected components were installed successfully. <a href=\"".$this->getView()->uri()."\">Return to the component list</a>.");
			}
			catch(Exception $e)
			{
				$messages[] = new MK_Message('success', "The following error occured: ".$e->getMessage());
			}
			
			$this->getView()->messages = $messages;
		}
	}

	public function sectionIndex()
	{
		
		$config = MK_Config::getInstance();

		$message_list = array();

		// Check: If running in demo mode
		if( $config->core->mode == MK_Core::MODE_DEMO )
		{
			$message_list[] = array(
				'type' => 'warning',
				'message' => "<strong>".$config->instance->name."</strong> is currently running in demonstration mode. Editing and deleting records is disabled. You can create records, however."
			);
		}
		
		// Check: json_encode function
		if( !function_exists('json_encode') )
		{
			$message_list[] = array(
				'type' => 'warning',
				'message' => "Cannot find the function 'json_encode' - This is used for some extended JavaScript functionality."
			);
		}
		
		// Check: cURL installed
		if( !function_exists('curl_init') )
		{
			$message_list[] = array(
				'type' => 'warning',
				'message' => "cURL library not installed - This is used for downloading resources located remotely."
			);
		}
		
		// Check: ZipArchive class
		if( !class_exists('ZipArchive') )
		{
			$message_list[] = array(
				'type' => 'warning',
				'message' => "Cannot find the 'ZipArchive' library - This is used for performing a backup."
			);
		}
		
		// Check: Can use ini_set
		if( !ini_set('memory_limit', '20M') )
		{
			$message_list[] = array(
				'type' => 'warning',
				'message' => "Cannot use 'ini_set'."
			);
		}
		else
		{
			ini_restore('memory_limit');
		}
		
		// Check: Uploads directory is writable
		if( !is_writable('../'.$config->site->upload_path) )
		{
			$message_list[] = array(
				'type' => 'warning',
				'message' => "Uploads folder is not writable '../".$config->site->upload_path."' Please chmod to 0777."
			);
		}
		
		// Check: Thumbs directory is writable
		if( !is_writable('../tpl/img/thumbs') )
		{
			$message_list[] = array(
				'type' => 'warning',
				'message' => "Thumbs folder is not writable '../tpl/img/thumbs/' Please chmod to 0777."
			);
		}
		
		// Check: Backups directory is writable
		if( !is_writable('resources/backups') )
		{
			$message_list[] = array(
				'type' => 'warning',
				'message' => "Backups folder is not writable 'resources/backups/' Please chmod to 0777."
			);
		}
		
		// Check: Restore directory is writable
		if( !is_writable('resources/restore') )
		{
			$message_list[] = array(
				'type' => 'warning',
				'message' => "Restore folder is not writable 'resources/restore/' Please chmod to 0777."
			);
		}
		
		// Check: Can use set_time_limit
		if( !set_time_limit(0) )
		{
			$message_list[] = array(
				'type' => 'warning',
				'message' => "Cannot use 'set_time_limit'."
			);
		}
		else
		{
			ini_restore('time_limit');
		}
		
		// Check: Get recent log entries
		$action_log_paginator = new MK_Paginator();
		$action_log_paginator
			->setPage( MK_Request::getParam('page', 1) )
			->setPerPage(20);

		$action_log_module = MK_RecordModuleManager::getFromType('action_log');
		$action_log_records = $action_log_module->getRecords($action_log_paginator);
		
		$paging_url = $this->getView()->uri( array('page' => '{page}') , false );
		$this->getView()->action_log_paginator = $action_log_paginator->render($paging_url);
		
		$action_log = array();
		
		foreach( $action_log_records as $action_log_record )
		{
			$user_link = $this->getView()->uri( array('controller' => 'users', 'section' => 'index', 'method' => 'edit', 'id' => $action_log_record->getUser()) );
			$action_log[] = new MK_Message('information', $action_log_record->getAction().($action_log_record->getUser() ? ' - <small>by <a href="'.$user_link.'">'.$action_log_record->renderUser().'</a> '.$action_log_record->renderDateTime().'</small>' : ''));
		}
		
		$this->getView()->action_log = $action_log;

		// Check: Backup no more than 30 days ago
		$backup_type = MK_RecordModuleManager::getFromType('backup');
		$backup_search = array(
			array('literal' => "`date_time` > DATE_SUB(NOW(), INTERVAL 30 DAY)")
		);

		$backup_records = $backup_type->searchRecords($backup_search);
		if( count($backup_records) == 0 )
		{
			$message_list[] = array(
				'type' => 'warning',
				'message' => "You have not <a href=\"".$this->getView()->uri(array('controller' => 'dashboard', 'section' => 'backup'))."\">backed up</a> within the last 30 days."
			);
		}
		
		if( count($message_list) === 0 )
		{
			$message_list[] = array(
				'type' => 'information',
				'message' => "There are currently no system notifications."
			);
		}
		
		$this->getView()->message_list = $message_list;
		
	}

	public function sectionEmailUsers()
	{
		$config = MK_Config::getInstance();

		$this->getView()->getHead()->prependTitle( 'Email Users' );
		
		if( $user_group = MK_Request::getQuery('group') )
		{
			
		}
		else
		{
			$user_module = MK_RecordModuleManager::getFromType('user');
			$user_group_module = MK_RecordModuleManager::getFromType('user_group');
			
			$_user_groups = $user_group_module->getRecords();
			$user_groups = array();
			
			foreach($_user_groups as $key => $group)
			{
				$total_users = (integer) $user_module->countRecords(array(
					array('field' => 'group', 'value' => $group->getId())
				));

				if( $total_users > 0 )
				{
					$group->setTotalUsers($total_users);
					$user_groups[$key] = $group;
				}
			}
			
			$this->getView()->groups = $user_groups;
		}

	}

	public function sectionFileManager()
	{
		$config = MK_Config::getInstance();

		$this->getView()->getHead()->prependTitle( 'File Manager' );

		$messages = array();
		$files = array();

		$files_to_delete = array();

		if( $get_file = MK_Request::getQuery('file-select') )
		{
			$files_to_delete[] = $get_file;
		}
		elseif( $post_files = MK_Request::getPost('file-select') )
		{
			$files_to_delete = $post_files;
		}

		if( count($files_to_delete) > 0 )
		{
			foreach($files_to_delete as $file)
			{
				$file = '../'.$config->site->upload_path.$file;
				$current_file = new MK_File($file);
				$current_file->delete();
			}
			$messages[] = new MK_Message('success', "The selected files were successfully deleted. <a href=\"".$this->getView()->uri()."\">Return to file list</a>.");
		}
		else
		{
		
			if( !is_readable('../'.$config->site->upload_path) )
			{
				$messages[] = new MK_Message('error', "The folder '../".$config->site->upload_path."' cannot be read. Please chmod this folder to 0777.");
			}
			else
			{
				$paginator = new MK_Paginator();
				$paginator
					->setPage( MK_Request::getParam('page', 1) )
					->setPerPage( 10 );
	
				$file_list = scandir('../'.$config->site->upload_path);
				
				foreach($file_list as $file)
				{
					if( $file === 'index.php' )
					{
						continue;
					}
					$file = '../'.$config->site->upload_path.$file;
					if($file != '.' && $file != '..' && !is_dir($file))
					{
						$files[] = new MK_File($file);
					}
				}
	
				$paginator->setTotalRecords( count($files) );
				$files = array_splice($files, $paginator->getRecordStart(), $paginator->getPerPage());
				$this->getView()->paginator = $paginator->render( $this->getView()->uri(array('page' => '{page}')) );
			}
	
			if( count($files) === 0 )
			{
				$messages[] = new MK_Message('information', "There are no files to display.");
			}
	
			$this->getView()->files = $files;
		}

		$this->getView()->messages = $messages;
	}

	public function sectionBackup(){

		$config = MK_Config::getInstance();

		$this->getView()->getHead()->prependTitle( 'Backup' );

		$form_settings = array(
			'attributes' => array(
				'class' => 'standard clear-fix large'
			)
		);

		$form_structure = array(
			'backup_confirm' => array(
				'type' => 'select',
				'options' => array(
					0 => 'No',
					1 => 'Yes'
				),
				'label' => 'Start backup?',
				'validation' => array(
					'boolean_true' => array()
				)
			),
			'backup_submit' => array(
				'type' => 'submit',
				'attributes' => array(
					'value' => 'Confirm'
				)
			)
		);

		$form = new MK_Form($form_structure, $form_settings);
		
		$this->getView()->form = $form->render();

		if($form->isSuccessful())
		{
			$messages = array();
			try
			{
				if(!class_exists('ZipArchive'))
				{
					throw new MK_Exception("Cannot find the 'ZipArchive' library - This is used for performing a backup.");
				}
				$this->getView()->setDisplayPath('dashboard/backup-processed');		
				ini_set('memory_limit', '200M');
				set_time_limit(0);
				$config = MK_Config::getInstance();
		
				// Create archive
				$zip = new MK_ZipArchive();
				$timestamp = time();
				$file_name = 'resources/backups/backup-'.$timestamp.'.zip';
				if ($zip->open($file_name, ZIPARCHIVE::CREATE) !== true)
				{
					throw new MK_Exception("Could not create backup file '$file_name'. Please ensure this directory is writable.");
				}
		
				// Backup files
				$message_list = array();
				
				$iterator  = new RecursiveIteratorIterator(new RecursiveDirectoryIterator("../tpl/uploads/"));
		
				foreach($iterator as $key=>$value)
				{
					$filename = basename($key);
					$zip->addFile($key, 'uploads/'.$filename) or $messages[] = new MK_Message('warning', "Cannot add file '$key' to archive - It may be corrupt or too large.");
				}
				
				// Backup Database
				$database_backup = 'resources/backups/database-'.$timestamp.'.sql';
				$mysql_dump = new MK_MySQLDump($config->db->name, $database_backup, false, false);
				$mysql_dump->doDump();
				$zip->addFile($database_backup, 'database/database.sql') or $messages[] = new MK_Message('warning', "Cannot add file 'database.sql' to archive - It may be corrupt or too large.");
				unlink($database_backup);
		
				// Create backup record
				$backup_type = MK_RecordModuleManager::getFromType('backup');
				$new_backup = MK_RecordManager::getNewRecord( $backup_type->getId() );
				$new_backup
					->setFile('admin/'.$file_name)
					->save();
				$this->getView()->backup = $new_backup;
		
				$messages[] = new MK_Message('success', 'Your backup has been successfully created. You can restore backups using the <a href="'.$this->getView()->uri( array('controller' => 'backups') ).'">backups module</a>.');

				// Close archive
				$zip->close();
			}
			catch(Exception $e)
			{
				$messages[] = new MK_Message('error', $e->getMessage());
			}

			$this->getView()->messages = $messages;
		}
		
	}

	public function sectionSettings(){
		$config = MK_Config::getInstance();

		$this->getView()->getHead()->prependTitle( 'Settings' );

		$form_settings = array(
			'attributes' => array(
				'class' => 'standard clear-fix large'
			)
		);

		$selected_fieldset = MK_Request::getParam('form', 'awesome-gallery');
		$this->getView()->selected_fieldset = $selected_fieldset;

		$form_structure = array();

		/* Website */
		if( $config->site->mode != MK_Core::MODE_DEMO )
		{
			$form_structure['site_mode'] = array(
				'type' => 'select',
				'options' => array(
					MK_Core::MODE_PRODUCT => 'Restricted',
					MK_Core::MODE_FULL => 'Full Access'
				),
				'label' => 'CMS Mode',
				'tooltip' => '<strong>Restricted</strong> - Includes all of features needed to use the gallery.<br /><strong>Full Access</strong> - Opens additional features for advanced customization.',
				'fieldset' => 'Awesome Gallery',
				'value' => $config->core->mode
			);
		}

        $form_structure['site_dev_mode'] = array(
			'type' => 'select',
			'options' => array(
				0 => 'No',
				1 => 'Yes'
			),
			'label' => 'Development Mode',
			'tooltip' => 'The LESS CSS file is compiled whenever it has been modified.',
			'fieldset' => 'Awesome Gallery',
			'value' => $config->site->dev_mode
		);
		
        $form_structure['site_url'] = array(
			'type' => 'text',
			'label' => 'Site URL',
			'fieldset' => 'Awesome Gallery',
			'validation' => array(
				'url' => array(),
				'instance' => array()
			),
			'value' => $config->site->url
		);
        
		$form_structure['site_email'] = array(
			'type' => 'text',
			'label' => 'Site Email Address',
			'fieldset' => 'Awesome Gallery',
			'validation' => array(
				'email' => array(),
				'instance' => array()
			),
			'value' => $config->site->email
		);

		$form_structure['site_timezone'] = array(
			'type' => 'select',
			'options' => MK_Utility::getTimezoneList(),
			'label' => 'Site Timezone',
			'fieldset' => 'Awesome Gallery',
			'validation' => array(
				'instance' => array()
			),
			'value' => $config->site->timezone
		);

		$form_structure['site_name'] = array(
			'type' => 'text',
			'label' => 'Site Name',
			'fieldset' => 'Awesome Gallery',
			'validation' => array(
				'instance' => array()
			),
			'value' => $config->site->name
		);

        $form_structure['site_caption'] = array(
			'type' => 'text',
			'label' => 'Site Caption/Tagline',
			'fieldset' => 'Awesome Gallery',
			'validation' => array(
				'instance' => array()
			),
			'value' => $config->site->caption
		);

        $form_structure['site_title'] = array(
			'type' => 'text',
			'label' => 'Meta Title',
			'tooltip' => 'Appended meta title.',
			'fieldset' => 'Awesome Gallery',
			'validation' => array(
				'instance' => array()
			),
			'value' => $config->site->title
		);

         $form_structure['site_desc'] = array(
			'type' => 'textarea',
			'attributes' => array(
				'class' => 'input-textarea-small'
			),
			'label' => 'Meta Description',
			'tooltip' => 'Meta description for your site.',
			'fieldset' => 'Awesome Gallery',
			'value' => $config->site->desc
		);
      
        $form_structure['site_logo'] = array(
			'type' => 'file_image',
			'label' => 'Logo',
            'tooltip' => 'Min height is 50px / x2 for retina.',
			'fieldset' => 'Awesome Gallery',
			'value' => $config->site->logo
		);
        
        $form_structure['site_logo_sticky'] = array(
			'type' => 'file_image',
			'label' => 'Sticky Menu Logo',
            'tooltip' => 'Min height is 20px / x2 for retina.',
			'fieldset' => 'Awesome Gallery',
			'value' => $config->site->logo_sticky
		);
        
        $form_structure['site_default_avatar'] = array(
			'type' => 'file_image',
            'label' => 'Default Avatar',
			'tooltip' => 'Square - min 225 x 225 / x2 for retina.',
			'fieldset' => 'Awesome Gallery',
			'value' => $config->site->default_avatar
		);
        


        $form_structure['site_home_per_page'] = array(
			'type' => 'select',
			'options' => array(
				12 => '12',
				24 => '24',
				36 => '36',
				48 => '48',
				60 => '60',
				72 => '72'
			),
			'label' => 'Home Content Per Page',
			'tooltip' => '# of images/videos to display on the home page',
			'fieldset' => 'Awesome Gallery',
			'value' => $config->site->home_per_page
		);
		
		
        $form_structure['site_enable_search'] = array(
			'type' => 'select',
			'options' => array(
				0 => 'No',
				1 => 'Yes'
			),
			'label' => 'Enable Search',
			'tooltip' => 'Display the search area',
			'fieldset' => 'Awesome Gallery',
			'value' => $config->site->enable_search
		);

        $form_structure['site_enable_comments'] = array(
			'type' => 'select',
			'options' => array(
				0 => 'No',
				1 => 'Yes'
			),
			'label' => 'Enable Comments',
			'tooltip' => 'Display comment area',
			'fieldset' => 'Awesome Gallery',
			'value' => $config->site->enable_comments
		);

        $form_structure['site_enable_reporting'] = array(
			'type' => 'select',
			'options' => array(
				0 => 'No',
				1 => 'Yes'
			),
			'label' => 'Enable Image Reporting',
			'tooltip' => 'Show button to let users report spammy uploads to admins.',
			'fieldset' => 'Awesome Gallery',
			'value' => $config->site->enable_reporting
		);

        $form_structure['site_enable_autoplay'] = array(
			'type' => 'select',
			'options' => array(
				0 => 'No',
				1 => 'Yes'
			),
			'label' => 'Enable Video Autoplay',
			'tooltip' => 'Autoplay settings do not effect Vines.',
			'fieldset' => 'Awesome Gallery',
			'value' => $config->site->enable_autoplay
		);

        $form_structure['site_enable_footer'] = array(
			'type' => 'select',
			'options' => array(
				0 => 'No',
				1 => 'Yes'
			),
			'label' => 'Enable Footer',
			'tooltip' => 'Display the footer area',
			'fieldset' => 'Awesome Gallery',
			'value' => $config->site->enable_footer
		);

        $form_structure['site_enable_page_loader'] = array(
			'type' => 'select',
			'options' => array(
				0 => 'No',
				1 => 'Yes'
			),
			'label' => 'Enable Loader',
			'tooltip' => 'Display the progress bar on every page load',
			'fieldset' => 'Awesome Gallery',
			'value' => $config->site->enable_page_loader
		);
		
		/*		
		$form_structure['site_log_actions'] = array(
			'type' => 'select',
			'options' => array(
				0 => 'No',
				1 => 'Yes'
			),
			'label' => 'Log Actions',
			'tooltip' => 'Log user actions such as deleting, editing and creating records.',
			'fieldset' => 'Awesome Gallery',
			'value' => $config->site->log_actions
		);
        */

        $form_structure['site_enable_tracking'] = array(
			'type' => 'select',
			'options' => array(
				0 => 'No',
				1 => 'Yes'
			),
			'label' => 'Enable Engage Tracking',
			'tooltip' => 'Help improve this product by sharing some of your public information with Engage.',
			'fieldset' => 'Awesome Gallery',
			'value' => $config->site->enable_tracking
		);
		
		
		        
		$form_structure['user_timeout'] = array(
			'type' => 'text',
			'label' => 'User login time',
			'validation' => array(
				'instance' => array(),
				'integer' => array()
			),
			'tooltip' => 'This field defines how long users will be signed in for (in seconds). After this period users will need to login again.<br>Default: 31556926',
			'fieldset' => 'Awesome Gallery',
			'value' => $config->site->user_timeout
		);


		/*
		$form_structure['site_valid_file_extensions'] = array(
			'type' => 'textarea',
			'label' => 'File extensions',
			'tooltip' => 'The following file types can be uploaded to the site. Separate with a comma \',\'.',
			'fieldset' => 'Awesome Gallery',
			'validation' => array(
				'instance' => array()
			),
			'value' => implode(', ', (array) $config->site->valid_file_extensions)
		);
		*/
		
		$form_structure['site_date_format'] = array(
			'type' => 'text',
			'label' => 'Date format',
			'tooltip' => ' Default: Y-m-d',
			'validation' => array(
				'instance' => array()
			),
			'fieldset' => 'Awesome Gallery',
			'value' => $config->site->date_format
		);
		$form_structure['site_time_format'] = array(
			'type' => 'text',
			'label' => 'Time format',
			'tooltip' => ' Default: H:i:s',

			'validation' => array(
				'instance' => array()
			),
			'fieldset' => 'Awesome Gallery',
			'value' => $config->site->time_format
		);
		$form_structure['site_template'] = array(
			'type' => 'select',
			'options' => array(),
			'label' => 'Template',
			'fieldset' => 'Awesome Gallery',
			'value' => $config->template.'/'.$config->template_theme
		);




        /* STYLING   
        $form_structure['site_style_primary'] = array(
			'type' => 'text',
			'label' => 'Header Color',
			'tooltip' => 'Color code: #XXXXXX or rgb(x,x,x)',
			'fieldset' => 'Styling',
			'validation' => array(
				'instance' => array()
			),
			'value' => $config->site->style_primary
		);
        $form_structure['site_style_secondary'] = array(
			'type' => 'text',
			'label' => 'Menu Text Color (Must be dark)',
			'tooltip' => 'Color code: #XXXXXX or rgb(x,x,x)',
			'fieldset' => 'Styling',
			'validation' => array(
				'instance' => array()
			),
			'value' => $config->site->site_style_secondary
		);
        $form_structure['site_style_boxradius'] = array(
			'type' => 'text',
			'label' => 'Menu Text Color (Must be dark)',
			'tooltip' => 'Ex. 3px -> Rounds the edges of many boxes.',
			'fieldset' => 'Styling',
			'validation' => array(
				'instance' => array()
			),
			'value' => $config->site->site_style_boxradius
		);
        $form_structure['site_style_boxshadow'] = array(
			'type' => 'text',
			'label' => 'Menu Text Color (Must be dark)',
			'tooltip' => 'Ex. 2px -> Adds a box shadow to some boxes.',
			'fieldset' => 'Styling',
			'validation' => array(
				'instance' => array()
			),
			'value' => $config->site->site_style_boxshadow
		);
		*/



		/* MEMBER PROFILES */
        $form_structure['site_profiles_enable_video'] = array(
			'type' => 'select',
			'options' => array(
				0 => 'No',
				1 => 'Yes'
			),
			'label' => 'Enable Video',
			'tooltip' => 'Allow users to display a personal video.',
			'fieldset' => 'User Profiles',
			'value' => $config->site->profiles->enable_video
		);

        $form_structure['site_profiles_enable_skills'] = array(
			'type' => 'select',
			'options' => array(
				0 => 'No',
				1 => 'Yes'
			),
			'label' => 'Enable Skills',
			'tooltip' => 'Allow users to add skills to their profile.',
			'fieldset' => 'User Profiles',
			'value' => $config->site->profiles->enable_skills
		);
		
        $form_structure['site_profiles_enable_gender'] = array(
			'type' => 'select',
			'options' => array(
				0 => 'No',
				1 => 'Yes'
			),
			'label' => 'Enable Gender',
			'tooltip' => 'Allow users to set a gender.',
			'fieldset' => 'User Profiles',
			'value' => $config->site->profiles->enable_gender
		);

		

		/* API KEYS */
		if($config->core->mode != MK_Core::MODE_DEMO)
		{

			// Facebook API
			$form_structure['site_bitly_login_id'] = array(
				'type' => 'text',
				'label' => 'Bitly Login ID',
				'fieldset' => 'API Keys',
				'value' => $config->site->bitly->login_id
			);
	
			$form_structure['site_bitly_app_key'] = array(
				'type' => 'text',
				'label' => 'Bitly App Key',
				'fieldset' => 'API Keys',
				'value' => $config->site->bitly->app_key
			);
			
			$form_structure['site_bitly_enabled'] = array(
				'type' => 'select',
				'options' => array(
					0 => 'No',
					1 => 'Yes'
				),
				'label' => 'Enable Bit.ly',
				'tooltip' => 'Create shortened links when social sharing? https://bitly.com/a/your_api_key',
				'fieldset' => 'API Keys',
				'value' => $config->site->bitly->enabled
			);


			// Facebook API
			$form_structure['site_facebook_app_id'] = array(
				'type' => 'text',
				'label' => 'Facebook App ID',
				'fieldset' => 'API Keys',
				'value' => $config->site->facebook->app_id
			);
	
			$form_structure['site_facebook_app_secret'] = array(
				'type' => 'text',
				'label' => 'Facebook App Secret',
				'fieldset' => 'API Keys',
				'value' => $config->site->facebook->app_secret
			);
			
			$form_structure['site_facebook_login'] = array(
				'type' => 'select',
				'options' => array(
					0 => 'No',
					1 => 'Yes'
				),
				'label' => 'Facebook Login',
				'tooltip' => 'Can users login to the site using their Facebook account?',
				'fieldset' => 'API Keys',
				'value' => $config->site->facebook->login
			);
	
			// Twitter API
			/*
            $form_structure['site_twitter_app_key'] = array(
				'type' => 'text',
				'label' => 'Twitter Consumer Key',
				'fieldset' => 'API Keys',
				'value' => $config->site->twitter->app_key
			);
	
			$form_structure['site_twitter_app_secret'] = array(
				'type' => 'text',
				'label' => 'Twitter Consumer Secret',
				'fieldset' => 'API Keys',
				'value' => $config->site->twitter->app_secret
			);
			
			$form_structure['site_twitter_login'] = array(
				'type' => 'select',
				'options' => array(
					0 => 'No',
					1 => 'Yes'
				),
				'label' => 'Twitter Login',
				'tooltip' => 'Can users login to the site using their Twitter account?',
				'fieldset' => 'API Keys',
				'value' => $config->site->twitter->login
			);*/

			// LinkedIn API
			$form_structure['site_linkedin_client_id'] = array(
				'type' => 'text',
				'label' => 'LinkedIn API Key',
				'fieldset' => 'API Keys',
				'value' => $config->site->linkedin->client_id
			);
	
			$form_structure['site_linkedin_client_secret'] = array(
				'type' => 'text',
				'label' => 'LinkedIn Secret Key',
				'fieldset' => 'API Keys',
				'value' => $config->site->linkedin->client_secret
			);
			
			$form_structure['site_linkedin_login'] = array(
				'type' => 'select',
				'options' => array(
					0 => 'No',
					1 => 'Yes'
				),
				'label' => 'LinkedIn Login',
				'tooltip' => 'Can users login to the site using their LinkedIn account?',
				'fieldset' => 'API Keys',
				'value' => $config->site->linkedin->login
			);

			// Yahoo API
			$form_structure['site_yahoo_client_id'] = array(
				'type' => 'text',
				'label' => 'Yahoo Consumer Key',
				'fieldset' => 'API Keys',
				'value' => $config->site->yahoo->client_id
			);
	
			$form_structure['site_yahoo_client_secret'] = array(
				'type' => 'text',
				'label' => 'Yahoo Consumer Secret',
				'fieldset' => 'API Keys',
				'value' => $config->site->yahoo->client_secret
			);
			
			$form_structure['site_yahoo_login'] = array(
				'type' => 'select',
				'options' => array(
					0 => 'No',
					1 => 'Yes'
				),
				'label' => 'Yahoo Login',
				'tooltip' => 'Can users login to the site using their Yahoo account?',
				'fieldset' => 'API Keys',
				'value' => $config->site->yahoo->login
			);
	
			// Windows Live API
			$form_structure['site_windowslive_client_id'] = array(
				'type' => 'text',
				'label' => 'Windows Live Client ID',
				'fieldset' => 'API Keys',
				'value' => $config->site->windowslive->client_id
			);
	
			$form_structure['site_windowslive_client_secret'] = array(
				'type' => 'text',
				'label' => 'Windows Live Client Secret',
				'fieldset' => 'API Keys',
				'value' => $config->site->windowslive->client_secret
			);
			
			$form_structure['site_windowslive_login'] = array(
				'type' => 'select',
				'options' => array(
					0 => 'No',
					1 => 'Yes'
				),
				'label' => 'Windows Live Login',
				'tooltip' => 'Can users login to the site using their Windows Live / Xbox Live account?',
				'fieldset' => 'API Keys',
				'value' => $config->site->windowslive->login
			);
	
			// Google API
			$form_structure['site_google_client_id'] = array(
				'type' => 'text',
				'label' => 'Google Client ID',
				'fieldset' => 'API Keys',
				'value' => $config->site->google->client_id
			);
	
			$form_structure['site_google_client_secret'] = array(
				'type' => 'text',
				'label' => 'Google Client Secret',
				'fieldset' => 'API Keys',
				'value' => $config->site->google->client_secret
			);
			
			$form_structure['site_google_login'] = array(
				'type' => 'select',
				'options' => array(
					0 => 'No',
					1 => 'Yes'
				),
				'label' => 'Google Login',
				'tooltip' => 'Can users login to the site using their Google account?',
				'fieldset' => 'API Keys',
				'value' => $config->site->google->login
			);
		}




        /* SOCIAL */  
        $form_structure['site_social_image_square'] = array(
			'type' => 'file_image',
			'label' => 'Square Banner For Social Sharing',
            'tooltip' => 'Recommended min. 250px x 250px.<br>php variable: $config->site->social->image_square',
			'fieldset' => 'Social',
			'value' => $config->site->social->image_square
		);
        $form_structure['site_social_image_wide'] = array(
			'type' => 'file_image',
			'label' => 'Wide Banner For Social Sharing',
            'tooltip' => 'Recommended min. 650px x 338px.<br>php variable: $config->site->social->image_wide',
			'fieldset' => 'Social',
			'value' => $config->site->social->image_wide
		);
        $form_structure['site_social_twitter'] = array(
			'type' => 'text',
			'label' => 'Twitter Handle',
			'tooltip' => 'ex. @username<br>php variable: $config->site->social->twitter',
			'fieldset' => 'Social',
			'value' => $config->site->social->twitter
		);
        $form_structure['site_social_facebook'] = array(
			'type' => 'text',
			'label' => 'Facebook Page',
			'tooltip' => 'ex. http://facebook.com/yourpage<br>php variable: $config->site->social->facebook',
			'fieldset' => 'Social',
			'validation' => array(
				'url' => array()
			),
			'value' => $config->site->social->facebook
		);
		$form_structure['site_social_instagram'] = array(
			'type' => 'text',
			'label' => 'Instagram Profile Page',
			'tooltip' => 'ex. http://instagram.com/yourpage<br>php variable: $config->site->social->instagram',
			'fieldset' => 'Social',
			'validation' => array(
				'url' => array()
			),
			'value' => $config->site->social->instagram
		);
		$form_structure['site_social_google_plus'] = array(
			'type' => 'text',
			'label' => 'Google+ Profile Page',
			'tooltip' => 'ex. http://plus.google.com/yourpage<br>php variable: $config->site->social->google_plus',
			'fieldset' => 'Social',
			'validation' => array(
				'url' => array()
			),
			'value' => $config->site->social->google_plus
		);

		$form_structure['site_social_flickr'] = array(
			'type' => 'text',
			'label' => 'Flickr Profile Page',
			'tooltip' => 'ex. http://flickr.com/yourpage<br>php variable: $config->site->social->flickr',
			'fieldset' => 'Social',
			'validation' => array(
				'url' => array()
			),
			'value' => $config->site->social->flickr
		);	
					
		$form_structure['site_social_pinterest'] = array(
			'type' => 'text',
			'label' => 'Pinterest Profile Page',
			'tooltip' => 'ex. http://pinterest.com/yourpage<br>php variable: $config->site->social->pinterest',
			'fieldset' => 'Social',
			'validation' => array(
				'url' => array()
			),
			'value' => $config->site->social->pinterest
		);
		$form_structure['site_social_youtube'] = array(
			'type' => 'text',
			'label' => 'Youtube Profile Page',
			'tooltip' => 'ex. http://youtube.com/yourpage<br>php variable: $config->site->social->youtube',
			'fieldset' => 'Social',
			'validation' => array(
				'url' => array()
			),
			'value' => $config->site->social->youtube
		);	

		$form_structure['site_social_vimeo'] = array(
			'type' => 'text',
			'label' => 'Vimeo Profile Page',
			'tooltip' => 'ex. http://vimeo.com/yourpage<br>php variable: $config->site->social->vimeo',
			'fieldset' => 'Social',
			'validation' => array(
				'url' => array()
			),
			'value' => $config->site->social->vimeo
		);	
		$form_structure['site_social_blog'] = array(
			'type' => 'text',
			'label' => 'Blog Site',
			'tooltip' => 'ex. http://yourblog.com<br>php variable: $config->site->social->blog',
			'fieldset' => 'Social',
			'validation' => array(
				'url' => array()
			),
			'value' => $config->site->social->blog
		);					
	
	
		
        /* Analytics */
        $form_structure['site_analytics'] = array(
			'type' => 'textarea',
			'attributes' => array(
				'class' => 'input-textarea-small'
			),
			'label' => 'Analytics Code',
			'tooltip' => 'Google Analytics (Stats Tracking) Code',
			'fieldset' => 'Analytics',
			'value' => $config->site->analytics
		);




        /* Ad Spaces */
        $form_structure['site_adhome728x90'] = array(
			'type' => 'textarea',
			'attributes' => array(
				'class' => 'input-textarea-medium'
			),
			'label' => 'Ad - Top Home 1',
			'tooltip' => 'Large leaderboard- 728x90',
			'fieldset' => 'Advertising',
			'value' => $config->site->adhome728x90
		);
        
        $form_structure['site_adhome242x90'] = array(
			'type' => 'textarea',
			'attributes' => array(
				'class' => 'input-textarea-medium'
			),
			'label' => 'Ad - Top Home 2',
			'tooltip' => 'Small leaderboard- 242x90',
			'fieldset' => 'Advertising',
			'value' => $config->site->adhome242x90
		);
        
        $form_structure['site_adsidebar160x600'] = array(
			'type' => 'textarea',
			'attributes' => array(
				'class' => 'input-textarea-medium'
			),
			'label' => 'Ad - Sidebar',
			'tooltip' => ' Skyscraper- 160x600',
			'fieldset' => 'Advertising',
			'value' => $config->site->adsidebar160x600
		);
        


        /* Emails */
		$form_structure['email_template'] = array(
			'type' => 'textarea',
			'attributes' => array(
				'class' => 'input-textarea-large'
			),
			'label' => 'Email Template',
			'tooltip' => 'All emails sent from the site will use this template. Use {email_content} to specify where you want the email text to appear.',
			'fieldset' => 'Email',
			'value' => $config->site->email_template
		);

        $form_structure['email_subject_signup'] = array(
			'type' => 'text',
			'label' => 'Welcome Email subject line (Sign Up)',
			'tooltip' => 'Subject line of email.',
			'fieldset' => 'Email',
			'validation' => array(
				'instance' => array()
			),
			'value' => $config->site->email_subject_signup
		);

		$form_structure['email_text_signup'] = array(
			'type' => 'textarea',
			'attributes' => array(
				'class' => 'input-textarea-small'
			),
			'label' => 'Welcome Email Text (Sign Up)',
			'tooltip' => 'This message will be sent to all new users that register for the site.',
			'fieldset' => 'Email',
			'value' => $config->site->email_text_signup
		);

         /*
		 $form_structure['email_text_verfification'] = array(
			'type' => 'textarea',
			'attributes' => array(
				'class' => 'input-textarea-small'
			),
			'label' => 'Email Text (Verification)',
			'tooltip' => 'This message will be sent to all new users that need to verify their email address after signing up.',
			'fieldset' => 'Email',
			'value' => $config->site->email_text_verfification
		);*/
 
 

		// Pull in custom component settings
		foreach($config->db->components as $component)
		{
			$module_settings = 'resources/components/'.$component.'/settings.php';
			if( is_file($module_settings) )
			{
				require_once $module_settings;
			}
		}
	
	
		
		/* DATABASE */
		if( $config->core->mode === MK_Core::MODE_FULL )
		{
			$form_structure['db_host'] = array(
				'type' => 'text',
				'label' => 'Host',
				'fieldset' => 'Database',
				'value' => $config->db->host
			);
			$form_structure['db_username'] = array(
				'type' => 'text',
				'label' => 'Username',
				'fieldset' => 'Database',
				'value' => $config->db->username
			);
			$form_structure['db_password'] = array(
				'type' => 'text',
				'label' => 'Password',
				'fieldset' => 'Database',
				'value' => $config->db->password,
				'attributes' => array(
					'type' => 'password'
				)
			);
			$form_structure['db_name'] = array(
				'type' => 'text',
				'label' => 'Database name',
				'fieldset' => 'Database',
				'value' => $config->db->name
			);
		}


		/* SAVE BUTTON */
		$form_structure['search_submit'] = array(
			'type' => 'submit',
			'attributes' => array(
				'value' => 'Save Changes'
			)
		);

		$handle = scandir('application/views');

		$components_core = array();
		$components_optional = array();
		foreach($handle as $template_folder)
		{
			if($template_folder!='.' && $template_folder!='..' && is_dir('application/views/'.$template_folder))
			{
				$template_details = simplexml_load_file('application/views/'.$template_folder.'/details.xml');
				$template_name = (string) $template_details->name;
				$form_structure['site_template']['options'][$template_name] = array();
				
				$theme_handle = scandir('application/views/'.$template_folder.'/themes');
				foreach( $theme_handle as $theme_folder )
				{
					if($theme_folder!='.' && $theme_folder!='..' && is_dir('application/views/'.$template_folder.'/themes/'.$theme_folder))
					{
						$theme_details = simplexml_load_file('application/views/'.$template_folder.'/themes/'.$theme_folder.'/details.xml');
						$form_structure['site_template']['options'][$template_name][$template_folder.'/'.$theme_folder] = (string) $theme_details->name.' - By '.(string) $theme_details->author;
					}
				}

			}
		}

		// Extract fieldsets for navigation tabs
		$fieldsets = array();
		foreach( $form_structure as $form_element_key => $form_element )
		{
			if( !empty($form_element['fieldset']) )
			{
				$fieldsets[MK_Utility::getSlug($form_element['fieldset'])] = $form_element['fieldset'];

				if( MK_Utility::getSlug($form_element['fieldset']) != $selected_fieldset )
				{
					unset($form_structure[$form_element_key]);
				}
			}
		}
		
		$this->getView()->fieldsets = $fieldsets;

		$form = new MK_Form($form_structure, $form_settings);

		if( $form->isSuccessful() )
		{
			$message = array();
			$config_data = array();
			
			$fields = $form->getFields();
			
			if($config->core->mode === MK_Core::MODE_FULL && $selected_fieldset == 'database' )
			{
				$config_data['db.host'] = $form->getField('db_host')->getValue();
				$config_data['db.name'] = $form->getField('db_name')->getValue();
				$config_data['db.username'] = $form->getField('db_username')->getValue();
				$config_data['db.password'] = $form->getField('db_password')->getValue();
			}

			if($config->core->mode != MK_Core::MODE_DEMO && $selected_fieldset == 'api-keys' )
			{
				$config_data['site.bitly.login_id'] = $form->getField('site_bitly_login_id')->getValue();
				$config_data['site.bitly.app_key'] = $form->getField('site_bitly_app_key')->getValue();
				$config_data['site.bitly.enabled'] = $form->getField('site_bitly_enabled')->getValue();

				$config_data['site.facebook.app_id'] = $form->getField('site_facebook_app_id')->getValue();
				$config_data['site.facebook.app_secret'] = $form->getField('site_facebook_app_secret')->getValue();
				$config_data['site.facebook.login'] = $form->getField('site_facebook_login')->getValue();
	
				/*$config_data['site.twitter.app_key'] = $form->getField('site_twitter_app_key')->getValue();
				$config_data['site.twitter.app_secret'] = $form->getField('site_twitter_app_secret')->getValue();
				$config_data['site.twitter.login'] = $form->getField('site_twitter_login')->getValue();
	*/
    
				$config_data['site.yahoo.client_id'] = $form->getField('site_yahoo_client_id')->getValue();
				$config_data['site.yahoo.client_secret'] = $form->getField('site_yahoo_client_secret')->getValue();
				$config_data['site.yahoo.login'] = $form->getField('site_yahoo_login')->getValue();
	
				$config_data['site.windowslive.client_id'] = $form->getField('site_windowslive_client_id')->getValue();
				$config_data['site.windowslive.client_secret'] = $form->getField('site_windowslive_client_secret')->getValue();
				$config_data['site.windowslive.login'] = $form->getField('site_windowslive_login')->getValue();
	
				$config_data['site.google.client_id'] = $form->getField('site_google_client_id')->getValue();
				$config_data['site.google.client_secret'] = $form->getField('site_google_client_secret')->getValue();
				$config_data['site.google.login'] = $form->getField('site_google_login')->getValue();
	
				$config_data['site.linkedin.client_id'] = $form->getField('site_linkedin_client_id')->getValue();
				$config_data['site.linkedin.client_secret'] = $form->getField('site_linkedin_client_secret')->getValue();
				$config_data['site.linkedin.login'] = $form->getField('site_linkedin_login')->getValue();
			}
            
            if( $selected_fieldset == 'analytics' )
			{
            
                $config_data['site.analytics'] = $form->getField('site_analytics')->getValue();

            }

            if( $selected_fieldset == 'user-profiles' )
			{
            
                $config_data['site.profiles.enable_gender'] = $form->getField('site_profiles_enable_gender')->getValue();
                $config_data['site.profiles.enable_video'] = $form->getField('site_profiles_enable_video')->getValue();
                $config_data['site.profiles.enable_skills'] = $form->getField('site_profiles_enable_skills')->getValue();

            }
                        
             if( $selected_fieldset == 'advertising' )
			{
            
                $config_data['site.adhome728x90'] = $form->getField('site_adhome728x90')->getValue();
                $config_data['site.adhome242x90'] = $form->getField('site_adhome242x90')->getValue();               
                $config_data['site.adsidebar160x600'] = $form->getField('site_adsidebar160x600')->getValue();
                
            }
            
            if( $selected_fieldset == 'email' )
			{
				$config_data['site.email_template'] = $form->getField('email_template')->getValue();
				
				$config_data['site.email_subject_signup'] = $form->getField('email_subject_signup')->getValue();
				$config_data['site.email_text_signup'] = $form->getField('email_text_signup')->getValue();
				
				//$config_data['site.email_text_verfification'] = $form->getField('email_text_verfification')->getValue();
				
			}

            if( $selected_fieldset == 'social' )
			{

                $config_data['site.social.twitter'] = $form->getField('site_social_twitter')->getValue();
                $config_data['site.social.facebook'] = $form->getField('site_social_facebook')->getValue();

                $config_data['site.social.instagram'] = $form->getField('site_social_instagram')->getValue();
                $config_data['site.social.flickr'] = $form->getField('site_social_flickr')->getValue();
                $config_data['site.social.google_plus'] = $form->getField('site_social_google_plus')->getValue();
                $config_data['site.social.youtube'] = $form->getField('site_social_youtube')->getValue();
                $config_data['site.social.vimeo'] = $form->getField('site_social_vimeo')->getValue();
                $config_data['site.social.blog'] = $form->getField('site_social_blog')->getValue();
                $config_data['site.social.pinterest'] = $form->getField('site_social_pinterest')->getValue();
            
                $config_data['site.social.image_wide'] = $form->getField('site_social_image_wide')->getValue();
                $config_data['site.social.image_square'] = $form->getField('site_social_image_square')->getValue();

            }
            
            /*
            if( $selected_fieldset == 'styling' )
			{

                $config_data['site.style_primary'] = $form->getField('site_style_primary')->getValue();
                $config_data['site.style_secondary'] = $form->getField('site_style_secondary')->getValue();
          
                $config_data['site.style_boxradius'] = $form->getField('site_style_boxradius')->getValue();
                $config_data['site.style_boxshadow'] = $form->getField('site_style_boxshadow')->getValue();

            }
            */
            
			if( $selected_fieldset == 'awesome-gallery' )
			{
				/*
				$valid_file_extensions = explode(',', $form->getField('site_valid_file_extensions')->getValue());
				$valid_file_extensions = array_map('trim', $valid_file_extensions);
				$valid_file_extensions = array_filter($valid_file_extensions);
				$valid_file_extensions = array_unique($valid_file_extensions);
				$config_data['site.valid_file_extensions'] = implode(',', $valid_file_extensions);
				*/
				
                $config_data['site.mode'] = $form->getField('site_mode')->getValue();
				$config_data['site.name'] = $form->getField('site_name')->getValue();
                $config_data['site.caption'] = $form->getField('site_caption')->getValue();
				$config_data['site.title'] = $form->getField('site_title')->getValue();
                $config_data['site.desc'] = $form->getField('site_desc')->getValue();
                
       
                $config_data['site.home_per_page'] = $form->getField('site_home_per_page')->getValue();
                $config_data['site.enable_page_loader'] = $form->getField('site_enable_page_loader')->getValue();
                $config_data['site.enable_footer'] = $form->getField('site_enable_footer')->getValue();
                $config_data['site.enable_search'] = $form->getField('site_enable_search')->getValue();
                $config_data['site.enable_comments'] = $form->getField('site_enable_comments')->getValue();
                $config_data['site.enable_reporting'] = $form->getField('site_enable_reporting')->getValue();
                $config_data['site.enable_autoplay'] = $form->getField('site_enable_autoplay')->getValue();
                $config_data['site.enable_tracking'] = $form->getField('site_enable_tracking')->getValue();

				//$config_data['site.log_actions'] = $form->getField('site_log_actions')->getValue();

				$config_data['user.timeout'] = $form->getField('user_timeout')->getValue();


                $config_data['site.dev_mode'] = $form->getField('site_dev_mode')->getValue();
				$config_data['site.email'] = $form->getField('site_email')->getValue();
				$config_data['site.timezone'] = $form->getField('site_timezone')->getValue();
				$config_data['site.url'] = $form->getField('site_url')->getValue();

                
				$config_data['site.logo'] = $form->getField('site_logo')->getValue();
                $config_data['site.logo_sticky'] = $form->getField('site_logo_sticky')->getValue();
                $config_data['site.default_avatar'] = $form->getField('site_default_avatar')->getValue();
                
                
				$config_data['site.date_format'] = $form->getField('site_date_format')->getValue();
				$config_data['site.time_format'] = $form->getField('site_time_format')->getValue();
				$config_data['site.template'] = $form->getField('site_template')->getValue();

			}

			foreach($fields as $field_key => $field_value)
			{
				$field_value_elements = explode('-', $field_key);
				$config_area = array_shift($field_value_elements);
				$config_section = array_shift($field_value_elements);
				$config_node = array_shift($field_value_elements);

				if($config_area === 'extensions')
				{
					$config_data["extensions.".$config_section.".".$config_node] = $field_value->getValue();
				}
			}

			if($config->core->mode === MK_Core::MODE_DEMO)
			{
				$messages[] = new MK_Message('warning', 'Settings cannot be updated as <strong>'.$config->instance->name.'</strong> is running in demonstration mode.');
			}
			else
			{
				$messages[] = new MK_Message('success', 'Your settings have been updated. <a href="'.$this->getView()->uri( array('controller' => 'dashboard', 'section' => 'settings', 'form' => $selected_fieldset) ).'">Make more changes</a> or <a href="'.$this->getView()->uri( array('controller' => 'dashboard', 'section' => 'index') ).'">return to the dashboard</a>.');
				MK_Utility::writeConfig($config_data);
			}

			$this->getView()->messages = $messages;
			$this->getView()->setDisplayPath('dashboard/settings-processed');		
		}
		else
		{
			$this->getView()->form = $form->render();
		}
		
	}

}

?>